unit ConnectMWO;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Menus, Vcl.ComCtrls,
  Registry, TlHelp32, MWOffice_TLB, Vcl.ExtCtrls, Vcl.StdCtrls;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    FileMenu: TMenuItem;
    Exit1: TMenuItem;
    StatusBar1: TStatusBar;
    Timer1: TTimer;
    Memo1: TMemo;
    Save: TMenuItem;
    SaveDialog1: TSaveDialog;
    MWOMenu: TMenuItem;
    ReadSchematics: TMenuItem;
    CreateSchematic: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure SaveClick(Sender: TObject);
    procedure ReadSchematicsClick(Sender: TObject);
    procedure CreateSchematicClick(Sender: TObject);
  private
    procedure TryConnect;
  public
    { Public declarations }
    MWOffice: IMWOffice;  // our interface to MWO
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}



// ====================== begin utilities =======================

function ExistClassID(const ClassID :string): Boolean;
// check if registry key exists
var
    Reg: TRegistry;
begin
 try
     Reg := TRegistry.Create;
   try
     Reg.RootKey := HKEY_CLASSES_ROOT;
     Result      := Reg.KeyExists(Format('CLSID\%s',[ClassID]));
   finally
     Reg.Free;
   end;
 except
    Result := False;
 end;
end;

function processExists(exeFileName: string): Boolean;
var
  ContinueLoop: BOOL;
  FSnapshotHandle: THandle;
  FProcessEntry32: TProcessEntry32;
begin
  FSnapshotHandle := CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
  FProcessEntry32.dwSize := SizeOf(FProcessEntry32);
  ContinueLoop := Process32First(FSnapshotHandle, FProcessEntry32);
  Result := False;
  while Integer(ContinueLoop) <> 0 do
  begin
    if ((UpperCase(ExtractFileName(FProcessEntry32.szExeFile)) =
      UpperCase(ExeFileName)) or (UpperCase(FProcessEntry32.szExeFile) =
      UpperCase(ExeFileName))) then
    begin
      Result := True;
    end;
    ContinueLoop := Process32Next(FSnapshotHandle, FProcessEntry32);
  end;
  CloseHandle(FSnapshotHandle);
end;

// ====================== end utilities =======================




procedure TMainForm.FormShow(Sender: TObject);
begin
  MWOffice := nil; // not connected yet
  TryConnect;      // call connection procedure
end;

procedure TMainForm.ReadSchematicsClick(Sender: TObject);
var
  aSchematic: ISchematic;
  m,n, o: Integer;
begin
  Memo1.Lines.Clear;

  if MWOffice<>nil then begin
    Memo1.Lines.Add('Project name: ' + MWOffice.Project.Name);

    if MWOffice.Project.Schematics.Count>0 then begin
      for n := 1 to MWOffice.Project.Schematics.Count do begin
        Memo1.Lines.Add(' Schematic name: ' + MWOffice.Project.Schematics[n].Name);
        Memo1.Lines.Add('   Number of ports: ' + IntToStr(MWOffice.Project.Schematics[n].Ports.Count));
        Memo1.Lines.Add('   Number of elements: ' + IntToStr(MWOffice.Project.Schematics[n].Elements.Count));
        if MWOffice.Project.Schematics[n].Elements.Count>0 then begin
          for m := 1 to MWOffice.Project.Schematics[n].Elements.Count do begin
            Memo1.Lines.Add('     Element #' + IntToStr(m) + ': ' + MWOffice.Project.Schematics[n].Elements[m].Name );
            for o := 1 to MWOffice.Project.Schematics[n].Elements[m].Parameters.Count do begin
               Memo1.Lines.Add('            ' + MWOffice.Project.Schematics[n].Elements[m].Parameters[o].Name +  ' = ' +  MWOffice.Project.Schematics[n].Elements[m].Parameters[o].ValueAsString);
            end;
          end;
        end;

     end;
    end else begin
      Memo1.Lines.Add('    No schematics in project');
    end;
  end;
end;

procedure TMainForm.SaveClick(Sender: TObject);
begin
  if SaveDialog1.Execute then begin
    Memo1.Lines.SaveToFile(SaveDialog1.FileName);
  end;
end;

procedure TMainForm.CreateSchematicClick(Sender: TObject);
var
  NewSchematic: ISchematic;
  NewElement: IElement;
  aParameter: IParameter;
  x,y : Integer;
begin
  if MWOffice<>nil then begin
    NewSchematic := MWOffice.Project.Schematics.Add('My new schematic');

    x := 0;
    y := 0;
    NewElement := NewSchematic.Elements.Add('MLIN', x, y, 0, false, '');
    NewElement.Parameters.Item['L'].ValueAsDouble := 5e-3;   // set length to 5mm
    NewElement.Parameters.Item['W'].ValueAsDouble := 2e-3;   // set length to 5mm

    // place next element in schematic at node 2 of the previous element
    x:= NewElement.Nodes.Item[2].x;
    y:= NewElement.Nodes.Item[2].y;
    NewElement   := NewSchematic.Elements.Add('RES', x, y, 0, false, '');

    x:= NewElement.Nodes.Item[2].x;
    y:= NewElement.Nodes.Item[2].y;
    NewElement   := NewSchematic.Elements.Add('MLIN', x, y, 0, false, '');

  end;
end;

procedure TMainForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.FormHide(Sender: TObject);
begin
  // end connection to MWO
  if MWOffice <> nil then MWOffice.Dispose;
end;


procedure TMainForm.TryConnect;
var
  timeout: integer;
begin

  if MWOffice = nil then begin   // if MWO is not yet connected, try to connect now
    // check if MWO class is registered on this system
    if ExistClassID('{7C8F18AB-77AD-49F5-98D1-183734AE64A7}') then begin
      // connect to MWO
      MWOffice := CoMWOApplication.Create;

      timeout := 500;  // timeout in millisecond before we consider MWO start failed
      repeat
        Sleep(10);
        timeout := timeout-10;
      until ((processExists('MWOffice.exe')) or (timeout=0));

      // we should not get here, but just in case MWO can't be started.
      if timeout=0 then MWOffice:= nil;

       // show connection status in status bar
      if MWOffice <> nil then StatusBar1.Panels[0].Text := 'Connected to MWO'
                         else StatusBar1.Panels[0].Text := 'Not connected to MWO';

    end else begin
      // MWO class not found in registry
      StatusBar1.Panels[0].Text := 'MWO is not installed on this computer';
    end;

  end;
end;

procedure TMainForm.Timer1Timer(Sender: TObject);
begin
  // check if MWO connection still alive. If not, set pointer to nil and try to reconnect.
  if NOT (processExists('MWOffice.exe')) then begin
     MWOffice := nil;
     StatusBar1.Panels[0].Text := 'Disconnected from MWO';
  end;
end;

end.
